package javamath.hasse.latdraw;
/* LatDrawWin.java (c) Ralph Freese 96/08/10

The bottons are controlled from their panel but for the improveButton,
labelled "Next", it would be better if I extended the Button class
to ImproveButton and controlled the action form there since I could
then do a disable(). Under the present scheme I need to keep a boolean,
improving, to tell if the button should be active. (I still call
enable() and disable() since they make it grep.

I experimented with popup message (Dialog) windows but first you
need to find the parent frame which is a little tricky for netscape.
The class ralph.util.FindParentFrame does this but netscape makes 
such a big stink about the security that it is annoying so I think
I will use my message area instead.

Window layout. the window hierarchy is a little complicated so it is
nice to have a diagram:

      LatFrame extends Frame
			     /         \
        MenuBar      LatDrawWin exents Applet
				    /     |        \
 LatDrawPanel  Bottom   LatChoose   all extend Panel
    Center       South     East
                                        /   \
                            ForceControls  ControlsAndMessage extends   Panel
                              North	             South
                                            /      |     \
                                      Controls  CountDown  MessageArea
                                       Panel      Canvas     Canvas
                                       North      Center     South
                                         |
                                     several buttons

The MessageArea is changed into an URLPanel (and back) by manipulating
the ControlsAndMessage panel.

Note it would be a little simpler to have all of the stuff below
ControlsAndMessage directly below it using a FlowLayout(5,1).

Most of the important variables are held in LatDrawPanel.


*/

import javamath.hasse.config.*;
import javamath.hasse.lattice.*;
import javamath.hasse.util.*;
import javamath.server.proxystore.*;
import java.awt.*;
import java.applet.*;
import java.io.*;
import java.util.*;
import java.net.URL;




public class LatDrawWin extends Applet {


	/** Configuration: PROXY_STORE is the url alias of the 
	* 	ProxyStore on this server.
	*/

  boolean isStandalone = false;
  String file;
  LatDrawPanel dp;
  MessageArea messageArea;
  Controls controls;
  String[][] messages = {
    {"Initial Diagram",
    "This is the initial positioning of the points in three space.",
			"Press Next when you are ready.",
    "You can also rotate it at any time.",
		"Mouse over vertex for label"},
	
    {"Phase I: Strong Repulsive Forces",
    "This is the phase where the replusive force is strong.",
    "Press Next to get the phase where the attractive force is strong.",
    "(The attraction and replusion sliders will only work after Phase III.)",
		"Mouse over vertex for label"},

    {"Phase II: Strong Attractive Forces",
    "This is the phase where the attrative force is strong.",
    "Press Next to get the phase where the forces are more balanced.",
    "(The attraction and replusion sliders will only work after Phase III.)",
		"Mouse over vertex for label"},

    {"Phase III: Balanced Forces",
    "In this final phase the forces are balanced.",
    "Press one of the rotate buttons to get a better picture.",
    "The Next button will now apply the forces determined by the sliders.",
		"Mouse over vertex for label"}};

		public String[][] getParameterInfo()
		{
			String[][] info = {
				// Parameter Name     Kind of Value   Description
				{"inputType",     "String",     "can be url or cookie"},
				{"inputKey",     "String",     "the url or cookie"}
				};

			return info;
		}




  public void start() {
    setLayout(new BorderLayout());
    //boolean getFile = false;
    boolean badOrderedSet = false;
    int reps = Diagram.numberOfIterations;
    // I am setting this to null since otherwise the compiler thinks it may
    // not be initialized.
    Diagram diagram = null;
    String str;
    InputLattice in = null;

		/* When we're coming in through main */
		if (isStandalone && (file != null)) // in this case we look for a file
		{
			try 
			{
				in = new InputLattice(file, InputLattice.FILE);
			} 
			catch (FileNotFoundException e) 
			{
				System.out.println(file + ": file not found.");
				badOrderedSet = true;
			} 
			catch (IOException e) 
			{
				throw new InternalError(e.toString());
			}
		}
		// in this case we look for a url or cookie - came as parameter to applet

		String inputType = getParameter("inputType");
		String inputKey = getParameter("inputKey");
		if (!isStandalone && (inputType!= null) && (inputKey!=null))
		{

			if (new String(inputType).equals(new String("url")))
			{

				try 
				{
					in = new InputLattice(inputKey, InputLattice.URL);
				} 
				catch (IOException e) 
				{
					throw new InternalError(e.toString());
				}
			} 
			else // inputType == cookie
			{
				URL documentBase = this.getDocumentBase();

				try
				{
					URL proxyStoreURL = new URL("http", documentBase.getHost(),
					documentBase.getPort(),"/"+ Config.PROXY_STORE);
					ProxyStore proxyStore = new ProxyStore(proxyStoreURL.toString());

					in = (InputLattice) proxyStore.get(inputKey);
				}
        catch (Exception e)
        {
           System.err.println("Exception: " + e.getMessage() + "\n");
           e.printStackTrace();
        }

			}
		}

    if (in != null) 
		{

			if (! badOrderedSet) 
			{
				try 
				{
					dp = new LatDrawPanel(new Diagram(in));
				} 
				catch (NonOrderedSetException e) 
				{
					System.out.println(e.getMessage());
					/* Couldn't make sense of the input, just proceed as though it 
					** weren't there.
					*/
					dp = new LatDrawPanel();
				}
			}
		}
		else
		{
			dp = new LatDrawPanel();
		}
		
    dp.latDrawWin = this;


    //textarea = new TextArea(messages[0],5,60);
    messageArea = new MessageArea(messages);
    messageArea.resize(dp.bounds().width,120);
    controls = new Controls(dp, messageArea);
    dp.controls = controls;
    add("Center", dp);
    CountDown cd = new CountDown();
    dp.countDown = cd;
    cd.resize(dp.bounds().width,4);
    //add("South", new ControlsAndMessage(controls, messageArea, cd));
    add("South", new Bottom(dp, controls, messageArea, 
                             cd, new GetURLPanel(dp)));
    LatChooser latChooser = new LatChooser(dp,controls,messageArea);
    dp.latChooser = latChooser;
    add("East", latChooser);
		this.repaint();

//Frame fr = FindParentFrame.getFrame(this);
//InfoDialog info = new InfoDialog(fr, "Testing", "This is a test.");
//info.show();

  }

  // This is to stop the running thread and free the resources.
  // Since the improveThread only runs a few seconds, don't worry about it. 
  // Since run isn't implemented, these don't work (?)

  public void stop() {
    dp.rotateStop();		
    controls.setButtonLabels();
  }

  public static void main(String args[]) {
    Frame f = new LatFrame("LatDraw");
    LatDrawWin latdrawwin = new LatDrawWin();
    latdrawwin.isStandalone = true;
    if (args.length != 0) latdrawwin.file = args[0];
    latdrawwin.init();
    latdrawwin.start();
    f.add("Center", latdrawwin);


//Frame fr = FindParentFrame.getFrame(latdrawwin);
//InfoDialog info = new InfoDialog(fr, "Testing", "This is a test.");
//info.show();
    
    MenuBar menubar = new MenuBar();
    f.setMenuBar(menubar);
    Menu file = new Menu("File");
    file.add(new MenuItem("Open"));
    file.add(new MenuItem("Quit"));
    menubar.add(file);
    Menu help = new Menu("Help");
    help.add(new MenuItem("About"));
    menubar.add(help);
    menubar.setHelpMenu(help);


    f.resize(500, 650);
    f.show();
  }
}

class LatDrawPanel extends Panel
{
  
	Vector allLats; //AS: remove artificial distinction between user and builtin
	static String[] latstr = {
       // N_5
       "(N5 (0 (b c)) (a (1)) (b (1)) (c (a)) (1 ()))",
 
       // D_2
       "(D2 (0 (a b c)) (a (d)) (b (d e)) (c (e)) (d (1)) (e (1)) (1 ()))",
 
       // FD(3)
       "(FD3 (0 (a b c)) (a (d e)) (c (d f)) (b (e f)) (d (x g)) (e (y g)) " +
       "(f (z g)) (g (h i j)) (h (k l)) (i (k m)) (j (l m)) (k (1)) " +
       "(l (1)) (m (1)) (x (h)) (y (i)) (z (j)) (1 ()))",
 
       // FL(1+3)
       "(FL1+3 (0 (2 1 )) (1 (4 )) (2 (4 3 )) (3 (8 6 )) (4 (5 )) (5 (8 7 )) " +
       "(6 (15 )) (7 (10 9 )) (8 (9 )) (9 (11 )) (10 (13 )) " +
       "(11 (13 12 )) (12 (15 14 )) (13 (14 )) (14 (16 )) " +
       "(15 (18 )) (16 (18 17 )) (17 (19 )) (18 (19 )) (19 ()))",
 
       // FM(3)
       "(FM3 (a ( b c ) ) (b ( d ) ) (d ( e ) ) (f ( w j ) ) (u ( l ) ) " +
       "(w ( u ) ) (v ( i ) ) (y ( x n ) ) (x ( v ) ) (1 ( ) ) (h ( 1 ) ) " +
       "(k ( 1 ) ) (g ( 1 ) ) (i ( g k ) ) (l ( g h ) ) (e ( h k ) ) " +
       "(c ( m d ) ) (j ( m u ) ) (n ( m v ) ) (m ( e i l ) ) " +
       "(o ( c n j ) ) (p ( o a ) ) (q ( o f ) ) (s ( o y ) ) (t ( p s ) ) " +
       "(z (s q ) ) (r ( p q ) ) (0 ( r z t ) ))",
 
 
       // BA(3)
       "(BA3 (0 (a b c)) (a (ab ac)) (b (ab bc)) (c (ac bc)) (ab (1)) " +
       "(ac (1)) (bc (1)) (1 ()))",
 
       // BA(4)
       "(BA4 (0 (1 2 4 8)) (1 (3 5 9)) (2 (3 6 10)) (3 (7 11)) (4 (5 6 12)) " + 
       "(5 (7 13)) (6 (7 14)) (7 (15)) (8 (9 10 12)) (9 (11 13)) " + 
       "(10 (11 14)) (11 (15)) (12 (13 14)) (13 (15)) (14 (15)) (15 ()))"};

  static float rotAngleDelta = (float)Math.PI/30;
  static int reps = Diagram.numberOfIterations;

  LatDrawWin latDrawWin;
  Diagram diagram; 
  OrderedSet poset;
  int size;
  int pointRadius = 5;
  int pointDiameter = 2 * pointRadius;
  int improvementCount;
  int phase1, phase2, phase3;
  float rotationAngle;
  Controls controls;
  CountDown countDown;
  ForceControls forceControls; 		// set when Bottom is initiated.
  ControlsAndMessage controlsAndMessage;
  LatChooser latChooser;
  Thread rotateThread;
  Thread improveThread;
  boolean rotating;
  boolean improving;
  //boolean erase = false;		// used to unpaint
  float attFactor;
  float repFactor;
  float att_I;				// attraction in Phase I, etc.
  float att_II;
  float att_III;
  float att_user;			// user set attraction.
  float rep_I;
  float rep_II;
  float rep_III;
  float rep_user;			// user set repulsion.

	/* Values for converting mouseEvents into vertex coordinates */
	int wd, ht; //store the nomimal dimensions of the panel
	int xorig, yorig; 

	// point labels
	String pntLabel = null;
	Graphics tg = null;
 	Point3D currentVertex = null;
	boolean justChanged = false; // from in to out or out to in
	int currentMouseX = 0;
	int currentMouseY = 0;


  public boolean mouseMove(Event e, int x, int y)
	{
		Point3D tmpv = vertexAt(x-xorig,y-yorig);
		// if we were on no vertex or are changing vertex...
		if (tmpv != null && ((currentVertex == null) || 
					!(new String(currentVertex.getLabel()).equals(tmpv.getLabel()) )))
		{
			currentVertex = tmpv;
			currentMouseX = x;
			currentMouseY = y;
			justChanged = true;
			repaint();
		} 
		// if we go from some vertex to no vertex ...
		else if ((tmpv == null) && (currentVertex != null))
		{
			currentVertex = null;
			justChanged = true;
			repaint();
		}
		else
		{
			justChanged = false;
		}
		return true;
  }

  public LatDrawPanel() 
	{
    setBackground(Color.white);

		allLats = new Vector();
		for (int i=0; i<latstr.length;i++)
		{
			try
			{
				allLats.addElement(new Diagram(
					new InputLattice(latstr[i],InputLattice.STRING)));
			}
			catch(java.io.FileNotFoundException e)
			{
				throw new InternalError(e.toString());
			}
			catch(javamath.hasse.lattice.NonOrderedSetException e)
			{
				throw new InternalError(e.toString());
			}
			catch(java.io.IOException e)
			{
				throw new InternalError(e.toString());
			}
		}
		setup((Diagram) allLats.elementAt(2));
	}

  public LatDrawPanel(Diagram d) {
		this();
		allLats.addElement(d);
    setup(d);
  }

	public Diagram stringToDiagram(String s) 
	{
		InputLattice in;
		Diagram d;

		/* AS: Assign to "in" the InputLattice resulting from the string "s".
		** "s" could be a filename, url or lattice encoded in LatDraw format.
		*/
		try 
		{
			in = new InputLattice(s, false);
		} 
		catch (FileNotFoundException e) 
		{
			throw new InternalError(e.toString());
    } 
		catch (IOException e) 
		{
			throw new InternalError(e.toString());
    }


		// AS: Convert the InputLattice to a Diagram
		try 
		{
			d = new Diagram(in);
		} 
		catch (NonOrderedSetException e) 
		{    
			// can't occur now; change later
			throw new InternalError(e.toString());
		}
		return d;
  }

  public void setup (String s) {
    setup(stringToDiagram(s));
  }

  public void setup (Diagram d) {
    rotating = false;
    improving = false;
    if (rotateThread != null) {
      rotateThread.stop();
      rotateThread = null;
    }
    if (improveThread != null) {
      improveThread.stop();
      improveThread = null;
    }
    diagram = d;
    float scale = diagram.center();
    diagram.normalizeCoords(scale);
    diagram.project2d(0);
    poset = d.getOrderedSet();
    size = d.size();
    rotationAngle = (float)0.0;
    improvementCount = 0;
    phase1 = reps + size + 30;
    phase2 = phase1 + reps + size;
    phase3 = phase2 + reps + size + 20;
    attFactor = d.attractionFactor;
    repFactor = d.repulsionFactor;
    att_user = attFactor;
    rep_user = repFactor;
    att_I = (float)0.5 * attFactor;
    att_II = (float)3.0 * attFactor;	// was 3.0
    att_III = (float)0.75 * attFactor;	// was 1.0
    rep_I = (float)3.0 * repFactor;
    rep_II = (float)0.5 * repFactor;	// was 0.5
    rep_III = (float)1.5 * repFactor;	// was 1.0

		this.repaint();
  }

  public synchronized void paint(Graphics g) {
    Rectangle r = this.bounds();


    int ht = (int)Math.round(0.95 * r.height);
    int wd = (int)Math.round(0.95 * r.width);


		this.wd=wd;
		this.ht=ht;
		this.xorig = (int)Math.round(0.025 * r.width);
		this.yorig = (int)Math.round(0.025 * r.height);

    g.translate(xorig,yorig);

    g.setColor(getForeground());
    paintCovers(g, wd, ht);
    paintElems(g, wd, ht, false);
		if (justChanged)
		{
			paintLabel(g, wd, ht);
		}

  }


  public void paintLabel(Graphics g, int wd, int ht) 
	{
    Rectangle r = this.bounds();
		if (currentVertex != null)
		{
			int textwd = 
				g.getFontMetrics().getMaxAdvance()*currentVertex.getLabel().length();
			int textht = getFont().getSize();

			g.setColor(getForeground());
			if (tg != null)
				tg.dispose();
			tg = g.create(Math.min(currentMouseX,r.width-textwd),Math.max(currentMouseY - 20,0), 
				(int) Math.round(textwd*1.2)+5, (int) Math.round(textht*1.2));
			tg.setColor(Color.yellow);
			tg.fillRect(0,0, textwd+5, (int) Math.round(textht*1.1));
			tg.setColor(getForeground());
			tg.drawRect(0,0, textwd+5, (int) Math.round(textht*1.1));
			tg.setPaintMode();
			tg.drawString(currentVertex.getLabel(), 5,textht);
		}
		else
		{
			g.setColor(getBackground());
			if (tg != null)
				tg.dispose();
			g.setColor(getForeground());
		}
	}

  public void paintCovers(Graphics g, int wd, int ht) {
    POElem elem;
    Point3D pt, pt2;
    Point3D[] vertices = diagram.getVertices();
    Vector elems = poset.univ();
    int x,y,x2,y2;
    for (int i = 0; i < size; i++) {
      elem = (POElem)elems.elementAt(i);
      Enumeration covs = ((Vector)elem.upperCovers()).elements();
      pt = vertices[i];
      x = convertx(pt.getX(), wd);
      y = converty(pt.getY(), ht);
      while (covs.hasMoreElements()) {
        pt2 = vertices[poset.elemOrder((POElem)covs.nextElement())];
        x2 = convertx(pt2.getX(), wd);
        y2 = converty(pt2.getY(), ht);
	g.drawLine(x,y,x2,y2);
      }
    }
  }


  public void paintElems(Graphics g,int wd, int ht, boolean erase) {
    int n = size;
    Point3D[] vertices = diagram.getVertices();
    for (int i = 0; i < n; i++) {
      if (erase) eraseElem(g, vertices[i], wd, ht);
      else paintElem(g, vertices[i], wd, ht);
    }
  }

	/* take the rectangle with upper left at (xmin,ymin) and
	** and return the first point centre is in that rectangle
	** under the current projection.
	*/
	public Point3D vertexAt(int xmid, int ymid)
	{
		int xmin = xmid-pointRadius;
		int ymin = ymid-pointRadius;
		int xmax = xmid + pointDiameter;
		int ymax = ymid + pointDiameter;
    Point3D[] vertices = diagram.getVertices();

		// Need the dimensions of this panel
    Rectangle r = this.bounds();

		for (int i = 0; i < vertices.length; i++) 
		{ 
			Point3D p = vertices[i];
			// get coordinates of the centre of the vertex
			int x = convertx(p.getX(),  wd);
			int y = converty(p.getY(), ht);

			if ((x >= xmin) && (x <= xmax) &&  (y >= ymin) && (y <= ymax))
			{
				return p;
			}
		}

		return null;

	}
  public void paintElem(Graphics g, Point3D p, int wd, int ht) {
    int x = convertx(p.getX(), wd);
    int y = converty(p.getY(), ht);
    g.setColor(this.getBackground());
    g.fillOval(x - pointRadius, y - pointRadius, pointDiameter, pointDiameter);
    g.setColor(this.getForeground());
    g.drawOval(x - pointRadius, y - pointRadius, pointDiameter, pointDiameter);
  }

  public void eraseElem(Graphics g, Point3D p, int wd, int ht) {
    int x = convertx(p.getX(), wd);
    int y = converty(p.getY(), ht);
    g.setColor(this.getBackground());
    g.fillOval(x - pointRadius, y - pointRadius, pointDiameter, pointDiameter);
  }

  int convertx(float x, int wd) {
    return (int)Math.round(x * wd);
  } 

  int converty(float y, int ht) {
    return (int)Math.round((1 - y) * ht);
  } 

  void rotateOnce () {
    rotationAngle += rotAngleDelta;
    if (rotationAngle > 2*Math.PI) rotationAngle -= 2*Math.PI;
    diagram.project2d(rotationAngle);
    repaint();
  }

  void rotateLeft () {
    rotationAngle -= rotAngleDelta;
    if (rotationAngle < -2*Math.PI) rotationAngle += 2*Math.PI;
    diagram.project2d(rotationAngle);
    repaint();
  }

  void rotateStart () {
    rotating = true;
    if (rotateThread != null && rotateThread.isAlive()) rotateThread.stop();
    rotateThread = new RotateThread(this);
    rotateThread.start();
  }

  void rotateStop () {
    rotating = false;
    if(rotateThread != null && rotateThread.isAlive()) {
      rotateThread.stop();
      rotateThread = null;
    }
  }



  void improveOnce () {
    if (improvementCount < phase1) {
      diagram.update(att_I, rep_I);
    } else if (phase1 <= improvementCount && improvementCount < phase2) {
      diagram.update(att_II, rep_II);
    } else if (phase2 <= improvementCount && improvementCount < phase3) {
      diagram.update(att_III, rep_III);
    } else {
      diagram.update(att_user, rep_user);
    }

    countDown.decrementCount();
    countDown.repaint();
    improvementCount++;
    float scale = diagram.center();
    diagram.normalizeCoords(scale);
    diagram.project2d(rotationAngle);
    repaint();
  }

}

class Bottom extends Panel {

  public Bottom(LatDrawPanel dp,Controls c, MessageArea ma, 
                  CountDown cd, GetURLPanel urlPanel) {
    setLayout(new BorderLayout());
    add("North", new ForceControls(dp));
    add("South", new ControlsAndMessage(dp,c,ma,cd,urlPanel));
		this.repaint();
  }
}

class ControlsAndMessage extends Panel {

  MessageArea ma;
  GetURLPanel urlPanel;
  Controls c;
  CountDown cd;

  public ControlsAndMessage (LatDrawPanel dp,Controls c, MessageArea ma, 
                               CountDown cd, GetURLPanel urlPanel) {
    this.ma = ma;
    this.urlPanel = urlPanel;
    this.c = c;
    this.cd = cd;
    dp.controlsAndMessage = this;
    setLayout(new BorderLayout());
    add("South",ma);
    //add("South",urlPanel);
    add("Center",cd);
    add("North",c);
		this.repaint();
  }

  void setURLMode () {
    //remove(ma);
    //add(urlPanel);
    removeAll();
    setLayout(new BorderLayout());
    add("North",c);
    add("Center",cd);
    add("South",urlPanel);
    validate();		// tells the layout manager to start over
  }

  void setMessageMode () {
    removeAll();
    setLayout(new BorderLayout());
    add("North",c);
    add("Center",cd);
    add("South",ma);
    validate();		// tells the layout manager to start over
  }

}

class CountDown extends Canvas {

  int count;
  int max;

  public CountDown () {
    setForeground(Color.black);
    count = 0;
    max = 1;
  }

  public void setMax (int m) {
    max = m;
  }

  public void setCount (int c) {
    count = c;
  }

  public void decrementCount () {
    count--;
  }

  public void resetParameters (int c) {
    count = c;
    max = c;
    repaint();
  }

  public void resetParameters () {
    count = 0;
    max = 1;
    repaint();
  }

  public void paint(Graphics g) {
    Rectangle r = this.bounds();
    int wd = r.width;
    int ht = r.height;
    int pos = wd*(max - count)/max;
    if (count <= 0) {
      g.setColor(Color.lightGray);
      //g.setColor(Color.cyan);
      g.fillRect(0, 0, wd, ht);
    } else {
      g.setColor(Color.red);
      g.fillRect(pos, 0, wd, ht);
      g.setColor(Color.green);
      g.fillRect(0, 0, pos, ht);
    }
  }

}

  // This used to extend Panel but then the resize (called in init) didn't work.
 
class MessageArea extends Canvas {

  static int bigFontSize = 24;
  static int smallFontSize = 16;
  static int tinyFontSize = 12;
  //static Font f = new Font("TimesRoman",Font.ITALIC+Font.BOLD,fontSize);
  static Font smallFont = new Font("Helvetica",Font.PLAIN,smallFontSize);
  static Font tinyFont = new Font("Helvetica",Font.PLAIN,tinyFontSize);
  //static Font bigFont = new Font("Helvetica",Font.BOLD, bigFontSize);
  static Font bigFont = new Font("TimesRoman",Font.BOLD, bigFontSize);
  FontMetrics big_fm; 
  FontMetrics tiny_fm; 
  FontMetrics small_fm;

  String[][] messages;
  int currentMessage;

  public MessageArea(String[][] m) {
    big_fm = getFontMetrics(bigFont);
    small_fm = getFontMetrics(smallFont);
    tiny_fm = getFontMetrics(tinyFont);
    messages = m;
    currentMessage = 0;
    setBackground(Color.lightGray);
  }

  public void setMessage(String[][] m) {
    messages = m;
  }

  public void nextMessage() {
    if (currentMessage < messages.length - 1) currentMessage++;
  }

  public void resetCurrentMessage() {
    currentMessage = 0;
    repaint();
  }

  public void paint(Graphics g) {
    if (currentMessage < messages.length) {
      Rectangle r = this.bounds();
      g.setColor(this.getBackground());
      g.fillRect(r.x, r.y, r.width, r.height);
      g.setColor(Color.blue);
      g.drawRect(1, 1, this.bounds().width - 3, this.bounds().height - 3);
      g.setColor(this.getForeground());
      g.setFont(bigFont);
      int y_pos = bigFontSize + 4;
      drawCenteredString(messages[currentMessage][0],g,big_fm,y_pos);

      g.setFont(smallFont);
      int y_delta = smallFontSize + 4;
      for(int i = 1; i < messages[currentMessage].length - 1; i++) {
        y_pos += y_delta;
        //g.drawString(messages[currentMessage][i],10,y_pos);
        drawCenteredString(messages[currentMessage][i],g,small_fm,y_pos);
      }
			// Do the last message line in tiny
      g.setColor(this.getForeground());
      g.setFont(tinyFont);
      y_pos += smallFontSize + 4;
      drawCenteredString(
				messages[currentMessage][messages[currentMessage].length-1],
				g,tiny_fm,y_pos);


    }
  }

  void drawCenteredString(String s, Graphics g, FontMetrics fm, int y) {
    int x = (this.size().width - fm.stringWidth(s))/2;
    g.drawString(s, x, y);
  }
}

class GetURLPanel extends Panel {

  static String [][] urlMessage = {{"Input Your Own Lattice", 
                 "Type in an URL with the lattice. See the directions below"}};
  LatDrawPanel target;
  MessageArea messageArea;
  Panel URL;
  Button okButton;
  Button cancelButton;
  Button clearButton;
  TextField textField;

  public GetURLPanel(LatDrawPanel target) {
    this.target = target;
    MessageArea messageArea = new MessageArea(urlMessage);
    messageArea.resize(target.bounds().width,61);
    textField = new TextField("http://",35);
    okButton = new Button("OK");
    cancelButton = new Button("Cancel");
    clearButton = new Button("Clear");
    Panel URLPanel = new Panel();
    URLPanel.setLayout(new FlowLayout()); 
    URLPanel.add(textField);
    URLPanel.add(okButton);
    URLPanel.add(cancelButton);
    URLPanel.add(clearButton);
    setLayout(new BorderLayout());
    //add("NORTH", okButton);
    add("Center", messageArea);
    add("South", URLPanel);
  }

  public boolean action(Event e, Object arg) {
    if (e.target == cancelButton) {
      target.controlsAndMessage.setMessageMode();
      return true;
    }
    if (e.target == clearButton) {
      textField.setText("");
      return true;
    }
    if (e.target == okButton || e.target == textField) {
      InputLattice in;
      Diagram diagram = null;
      try {
        in = new InputLattice(textField.getText(), InputLattice.URL);
      } catch (FileNotFoundException ex) {
        handleError(ex.getMessage() + " file not found.");
        return true;
      } catch (IOException ex) {
        handleError(ex.getMessage() + 
                 ": bad URL, or file not found, or empty file");
        return true;
      }
      try {
        diagram = new Diagram(in);
      } catch (NonOrderedSetException ex) {
        if (NonOrderedSetException.emptyError.equals(ex.getMessage())) {
          handleError(ex.getMessage() + " Or the file did not exist.");
        } else {
          handleError(ex.getMessage());
        }
        return true;
      }
      if (diagram != null) {
        if (target.allLats.size() == target.latChooser.max_lats) {
          shiftVector(target.allLats);
          target.allLats.setElementAt(diagram, 
                                target.latChooser.max_lats - 1);
        } else {
          target.allLats.addElement(diagram);
          target.latChooser.addChoice();
        } 
        target.setup(diagram);
        target.countDown.resetParameters();
        target.controls.improveButtonDark();
        target.controlsAndMessage.ma.resetCurrentMessage();
        target.controlsAndMessage.setMessageMode();
        target.repaint();
      }
      return true;
    }
    return false;
  }

  public void handleError(String err) {
    if (target.latDrawWin.isStandalone) {
      System.out.println(err);
    } else {
      target.latDrawWin.showStatus(err);
    }
  }

  /**
    * pop of the first element of a vector, move everything up one.
    */
  public void shiftVector(Vector v) {
    int k = v.size() - 1;
    for(int i = 0; i < k; i++) {
      v.setElementAt(v.elementAt(i+1),i);
    }
  }

}
  
    

class Controls extends Panel {
    LatDrawPanel target;
    MessageArea messageArea;
    Button improveButton;
    Button rotateButton;
    Button rotateRightButton;
    Button rotateLeftButton;
    Button inputButton;
    //Menu inputMenu;
// Button infoButton;

    public Controls(LatDrawPanel target, MessageArea ma) {
	this.target = target;
        messageArea = ma;
	setLayout(new FlowLayout());
	setBackground(Color.lightGray);
        improveButton = new Button("Next");
        add(improveButton);
        rotateButton = new Button("Rotate");
        add(rotateButton);
        rotateRightButton = new Button("Rotate >>");
        add(rotateRightButton);
        rotateLeftButton = new Button("Rotate <<");
        add(rotateLeftButton);
        inputButton = new Button("Input");
        add(inputButton);

	//inputMenu = new Menu("Input a Lattice");
        //MenuItem mi = new MenuItem("by URL");
        //inputMenu.add(mi);
        //add(inputMenu);

// infoButton = new Button("Info Test");
// add(infoButton);

		target.repaint();
    }

    // See not in LatDrawWin.java for the reason for deleting 
    // this repaint.

    public void improveButtonDark () {
      // improveButton.setForeground(Color.black);
      // //improveButton.repaint();
      // repaint();
      target.improving = false;
      improveButton.enable();
    }

    public void improveButtonFaded () {
      // improveButton.setForeground(Color.gray);
      // //improveButton.repaint();
      // repaint();
      target.improving = true;
      improveButton.disable();
    }

    public void setButtonLabels () {
      rotateButton.setLabel("Rotate");
      //improveButton.setLabel("Next");
      improveButtonDark();
      repaint();
    }
      

    public void paint(Graphics g) {
	Rectangle r = bounds();

	g.setColor(Color.lightGray);
	g.draw3DRect(0, 0, r.width, r.height, false);
        g.setColor(Color.green);
        g.drawRect(1,1,r.width - 3, r.height - 3);
    }

    public boolean action(Event e, Object arg) {
	if (e.target instanceof Checkbox) {
	    //target.setForeground(((Component)e.target).getBackground());
	} else if (e.target instanceof Choice) {
	    String choice = (String)arg;

	} else if (e.target == rotateButton) {
             if (target.rotating) {
               rotateButton.setLabel("Rotate");
               repaint();
               target.rotateStop();
             } else {
               rotateButton.setLabel("Stop");
               repaint();
               target.rotateStart();
             } 
					target.repaint();
					target.postEvent(new Event(target,Event.MOUSE_MOVE,null));
					// target.processEvent(new AWTEvent(target,));
        } else if (e.target == improveButton && !target.improving) {
          // target.improving = true;
          messageArea.nextMessage();
          messageArea.repaint();
          int c = target.improvementCount;
          int phase1 = target.phase1;
          int phase2 = target.phase2;
          int phase3 = target.phase3;
          if (c < phase1) {
            target.forceControls.setForces(target.att_I,target.rep_I);
            target.countDown.resetParameters(phase1);
          } else if (c < phase2) {
            target.forceControls.setForces(target.att_II,target.rep_II);
            target.countDown.resetParameters(phase2 - phase1);
          } else if (c < phase3) {
            target.forceControls.setForces(target.att_III,target.rep_III);
            target.countDown.resetParameters(phase3 - phase2);
          } else {
            target.att_user = target.forceControls.getAttraction();
            target.rep_user = target.forceControls.getRepulsion();
            target.countDown.resetParameters(40);
          }

          improveButtonFaded();

          Thread fixIt = new ImproveThread(target);
          target.improveThread = fixIt;
          fixIt.start();

				target.repaint();
        } else if (e.target == rotateRightButton) {
          target.rotateOnce();
        } else if (e.target == rotateLeftButton) {
          target.rotateLeft();
        } else if (e.target == inputButton) {
// here          
          target.controlsAndMessage.setURLMode();

        }

// {
// } else if (e.target == infoButton) {
// }
// Frame fr = FindParentFrame.getFrame(this);
// InfoDialog info = new InfoDialog(fr, "Testing", "This is a test.");
// info.show();


	target.repaint();
	return true;
    }

}

class ForceControls extends Panel {
  LatDrawPanel target;
  //Scrollbar attBar;
  //Scrollbar repBar;
  AttRepSlider attBar, repBar;
  Label attLabel;
  Label repLabel;
  int att;			// the real attraction * 300
  int rep;			// the real repulsion * 30
  
  public ForceControls (LatDrawPanel dp) {
    target = dp;
    dp.forceControls = this;
    setLayout(new FlowLayout());
    //attBar = new Scrollbar(Scrollbar.HORIZONTAL,30,120,10,120);
    //repBar = new Scrollbar(Scrollbar.HORIZONTAL,30,120,10,120);
    attBar = new AttRepSlider("attraction");
    repBar = new AttRepSlider("replusion");
    //attLabel = new Label("Attraction 30  ");
    //repLabel = new Label("Repulsion 30  ");
    attLabel = new Label("Attraction: ");
    repLabel = new Label("Repulsion: ");
    add(attLabel);
    add(attBar);
    add(repLabel);
    add(repBar);
  }

  public void setForces(float attraction, float repulsion) {
    att = Math.round((attraction/target.attFactor)*(float)30.0);
    rep = Math.round((repulsion/target.repFactor)*(float)30.0);
    attBar.setValue(att);
    repBar.setValue(rep);
    //attLabel.setText("Attraction " + att + "  ");
    //repLabel.setText("Repulsion " + rep + "  ");
    repaint();
  }

  public float getAttraction() {
    return (attBar.getValue() / (float)30.0) * target.attFactor;
  }

  public float getRepulsion() {
    return (repBar.getValue() / (float)30.0) * target.repFactor;
  }

/*
  public boolean handleEvent(Event e) {
    if (e.target == attBar) {
      int v = ((Scrollbar)e.target).getValue();
      att = v;
      attLabel.setText("Attraction " + v);
    } else if (e.target == repBar) {
      int v = ((Scrollbar)e.target).getValue();
      rep = v;
      repLabel.setText("Repulsion " + v);
    }
    return true;
  }
*/
}

class AttRepSlider extends Slider {
  ForceControls parent;
  String name;		// will be either attraction or replusion

/**
 * Construct a slide of this type
 * @param name Either attraction or replusion.
 */
  public AttRepSlider (String name) {
    super();
    this.name = name;
    SetBarColor(Color.white);
    SetMinimum(10);
    SetMaximum(120);
    SetValue(30);
  }


  public void Release () {
    if (name == "attraction") parent.att = GetValue();
    if (name == "replusion")  parent.rep = GetValue();
  }
  
}

class LatChooser extends Panel {
    LatDrawPanel target;
    Controls controls;
    MessageArea messageArea;
    CountDown countDown;
    CheckboxGroup grp;
		Vector cbxvec;
    int fixed_choices; 
    int user_choices = 0; 
    static final int max_lats = 9;
		

    public LatChooser (LatDrawPanel dp,Controls c, MessageArea ma) {
      target = dp;
			cbxvec = new Vector();
      fixed_choices = target.latstr.length;
      controls = c;
      messageArea = ma;
      countDown = dp.countDown;
      setBackground(Color.lightGray);
      setLayout(new GridLayout(fixed_choices + 1,1));
      grp = new CheckboxGroup();
			for (int i = 0; i < target.allLats.size();i++)
			{
				Diagram tmp=(Diagram) target.allLats.elementAt(i);
				cbxvec.addElement(new Checkbox(tmp.getName(), grp,false));
				add((Checkbox) cbxvec.elementAt(i));
			}

			
    }

    public void addChoice() {
      invalidate();
			String tmpname = 
				((Diagram) target.allLats.lastElement()).getName();

			if (tmpname == null)
			{
				tmpname = "User"+target.allLats.size();
			}

      add(new Checkbox(tmpname,grp,true));
      setLayout(new GridLayout(target.allLats.size(),1));
      validate();
      repaint();
    }
      
    public void paint(Graphics g) {
      Rectangle r = bounds();
      g.setColor(Color.lightGray);
      g.draw3DRect(0, 0, r.width, r.height, true);
      //g.setColor(Color.yellow);
      //g.drawRect(1,1,r.width - 3, r.height - 3);
      //g.drawRect(0,0,r.width, r.height);
    }

// also need to repaint the buttons and messages

    public boolean action(Event e, Object arg) {
      if (e.target instanceof Checkbox) {
        String lat = ((Checkbox)e.target).getLabel();
        int k = 2;
				//AS: map checkbox back to its index...
				for (int i=0; i<cbxvec.size();i++)
				{
					if (((Checkbox)e.target) == cbxvec.elementAt(i))
					{
						k = i;
					}
				}
				target.setup((Diagram)target.allLats.elementAt(k));
        target.repaint();
        controls.improveButtonDark();
        controls.setButtonLabels();
        messageArea.resetCurrentMessage();
        countDown.resetParameters();
      }
      return true;
    }

}

class ImproveThread extends Thread {

  LatDrawPanel dp;
  private int phase1, phase2, phase3;
  static private int improveDelay = 150;

  public ImproveThread(LatDrawPanel dp) {
    this.dp = dp;
    phase1 = dp.phase1;
    phase2 = dp.phase2;
    phase3 = dp.phase3;
  }

  public void run () {
    if (dp.improvementCount < phase1) {
      while (dp.improvementCount < phase1) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } else if (dp.improvementCount < phase2) {
      while (dp.improvementCount < phase2) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } else if (dp.improvementCount < phase3) {
      while (dp.improvementCount < phase3) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } else {
      for(int k = 0; k < 40; k++) {
        dp.improveOnce();
        try {
          Thread.sleep(improveDelay);
        } catch (InterruptedException e) { }
      }
    } 
    dp.controls.improveButtonDark();
  }


}
  
class RotateThread extends Thread {

  static private int rotateDelay = 150;

  LatDrawPanel dp;

  public RotateThread(LatDrawPanel dp) {
    this.dp = dp;
  }

  public void run() {
    while (true) {
      dp.rotateOnce();
      try {
        Thread.sleep(rotateDelay);
      } catch (InterruptedException e) { }
    }
  }

}

class LatFrame extends Frame {

  public LatFrame (String str) {
    super(str);
  }
  
/*
  public boolean action(Event e, Object arg) {
    if (e.target instanceof MenuItem) {
      String label = (String)arg;
      if (label.equals("Quit")) {
        System.exit(0);
        return true;
      }
    }
    return false;
  }
*/

  public boolean handleEvent(Event e) {
    switch (e.id) {
      case Event.WINDOW_DESTROY:
        System.exit(0);
        return true;
      case Event.ACTION_EVENT:
        if (e.target instanceof MenuItem) {
          if (((String)e.arg).equals("Quit")) {
            System.exit(0);
            return true;
          } 
        }
      default:
        return false;
    }
  }

}

