/*
 * Universal Algebra Calculator
 * Copyright (C) 2001 Ralph Freese (ralph@math.hawaii.edu)
 *                    Emil W. Kiss (ewkiss@cs.elte.hu)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 *
 * The project's pages are at
 *     http://www.cs.elte.hu/~ewkiss/software/uaprog
 *     http://www.math.hawaii.edu/~ralph/software/uaprog
 */

package uacalc;

import uacalc.UACalc;

/**
 * Handles progress related things. At the moment it has
 * three functions:
 * <ul>
 * <li>Allows to write logging information.
 * <li>Makes possible to interrupt calculations.
 * <li>Makes locking possible during calculations.
 * </ul>
 */
public class Progress{

/**
 * Append text to the results area {@link LoggingPane#ResultLog}.
 *
 * @param text The text to be appended.
 */
    public static void ResultLogAppend(String text) {
	UACalc.log.ResultLogAppend(text);
    }

/**
 * Append text to the progress area {@link LoggingPane#ProgressLog}.
 *
 * @param text The text to be appended.
 */
    public static void ProgressLogAppend(String text) {
	UACalc.log.ProgressLogAppend(text);
    }

/**
 * This function provides locking when a calculation is started.
 * It should be called when the calculation is started.
 * It returns false if another calculation is already being
 * performed. In this case, an error message is written.
 * It is the caller's responsibility not to start the calculation.
 * When the calculation is finished, {@link #ProgramFinished} should
 * be called.
 *
 * @return true if no calculation is being performed at the moment.
 *
 */
    public static boolean ProgramStart() {
	if (ProgramIsRunning) {
	    //Later we could do something like
	    //disabling menus, popping up dialogs, etc.
	    System.out.println(
		"A computation is already running, cannot start another.");
	    return false;
	}
	ProgramIsRunning = true;
	return true;
    }

/**
 * This function removes locking when a calculation is finished.
 * It is safe to call this function even if a lock is not present.
 * See {@link #ProgramStart} for details.
 */
    public static void ProgramFinished() {
	ProgramIsRunning = false;
    }

/**
 * True if a calculation is being performed. See {@link #ProgramStart}.
 */
    private static boolean ProgramIsRunning = false;

/**
 * Variable to check if a calculation is to be interrupted.
 * The Stop menu in {@link Compute} sets this to true.
 * Therefore, if you notice during calculation that this
 * variable is true, then you should abort the calculation.
 * At the beginning of the calculation, you should set this to false.
 * Later we shall have a better mechanism for this, similar to
 * {@link #ProgramStart}.
 */
    public static boolean StopProgram;

}
