package uacalc.alg.algedit;

import uacalc.alg.Algebra;
import uacalc.alg.Operation;
import uacalc.alg.opedit.ValueEditor;
import uacalc.config.Configure;
import uacalc.util.ewkGetNumberDialog;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class AlgebraEditor extends JFrame {

    public AlgebraEditor(JFrame parent ) {

//initialize local variables
	m_UACalc = parent;

//Set up menus
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);

//File menu
        JMenu FileMenu = new JMenu("File");
	FileMenu.setMnemonic(KeyEvent.VK_F);
        menuBar.add(FileMenu);

//New
        NewMenuItem = new JMenuItem("New", KeyEvent.VK_N);
	FileMenu.add(NewMenuItem);
	NewMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));

	NewMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnNew();
		}
	    });

//Open
        OpenMenuItem = new JMenuItem("Open", KeyEvent.VK_O);
        FileMenu.add(OpenMenuItem);
        OpenMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
 
        OpenMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnOpen();
                }
            });
 
        FileMenu.addSeparator();
                                                                                                             
//Save
        SaveMenuItem = new JMenuItem("Save", KeyEvent.VK_S);
	FileMenu.add(SaveMenuItem);
	SaveMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));

	SaveMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnSave();
		}
	    });

//SaveAs
        SaveAsMenuItem = new JMenuItem("Save As...", KeyEvent.VK_V);
        FileMenu.add(SaveAsMenuItem);
        SaveAsMenuItem.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
 
        SaveAsMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    OnSaveAs();
                }
            });
 
        FileMenu.addSeparator();
                                                                                                             
//Close
        CloseMenuItem = new JMenuItem("Close", KeyEvent.VK_C);
	FileMenu.add(CloseMenuItem);
	CloseMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));

	CloseMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnClose();
		}
	    });

//Operations menu
        JMenu OperationsMenu = new JMenu("Operations");
	OperationsMenu.setMnemonic(KeyEvent.VK_P);
        menuBar.add(OperationsMenu);


//Edit
        EditMenuItem = new JMenuItem("Edit values", KeyEvent.VK_E);
	OperationsMenu.add(EditMenuItem);
	EditMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));

	EditMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnEdit();
		}
	    });

//Add
        AddMenuItem = new JMenuItem("Add operation", KeyEvent.VK_A);
	OperationsMenu.add(AddMenuItem);
	AddMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));

	AddMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnAdd();
		}
	    });

//Remove
        RemoveMenuItem = new JMenuItem("Remove operation", KeyEvent.VK_R);
	OperationsMenu.add(RemoveMenuItem);
	RemoveMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));

	RemoveMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnRemove();
		}
	    });


//Switch
        SwitchMenuItem = new JMenuItem("Switch operations", KeyEvent.VK_W);
	OperationsMenu.add(SwitchMenuItem);
	SwitchMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));

	SwitchMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnSwitch();
		}
	    });

//Change Arity
        ChangeMenuItem = new JMenuItem("Change arity", KeyEvent.VK_H);
	OperationsMenu.add(ChangeMenuItem);
	ChangeMenuItem.setAccelerator(
	    KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));

	ChangeMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    OnChange();
		}
	    });


//Set up window components

	pane = getContentPane();

	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));

	leftPane = new JPanel();
	leftPane.setLayout(new BoxLayout(leftPane, BoxLayout.Y_AXIS));
	leftPane.setAlignmentY(Component.TOP_ALIGNMENT);
	Configure.createBorder(leftPane);

	rightPane = new JPanel();
	rightPane.setLayout(new BoxLayout(rightPane, BoxLayout.Y_AXIS));
	rightPane.setAlignmentY(Component.TOP_ALIGNMENT);
	Configure.createBorder(rightPane);

	algebraPane = new AlgebraDisplayerPane(leftPane, rightPane);

//List of Operations
	opList = new OpListTable();
	opList.setAlignmentX(Component.LEFT_ALIGNMENT);
        Configure.createBorder(opList);



	rightPane.add(opList);

	JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

        panel.add(leftPane);
        panel.add(rightPane);
	JScrollPane scroll = new JScrollPane(panel);
	pane.add(scroll);
 
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    OnClose();
                }
            });



//No algebra originally.

	pack();
	m_editedAlgebra = new EditedAlgebra(this, SaveMenuItem);
	hideAlgebra();

        setTitle("Algebra Editor");
	parent.setEnabled(false);
        setVisible(true);
    }


////////////////////////////////////
//Menu callbacks:
////////////////////////////////////

//File menu callbacks

    public void OnNew() {

	int n = m_editedAlgebra.createNewAlgebra();
	if( n == 0 ) return;
	if( n == 1 ) {
	    hideAlgebra();
	    return;
	}
	ShowAlgebra();
    }

    public void OnOpen() {

	if( !m_editedAlgebra.openAlgebra() ) return;
	ShowAlgebra();
    }

    public void OnSave() {

	boolean nameChanged =
	    m_editedAlgebra.prepareWriteAlgebra();
	m_editedAlgebra.WriteAlgebra();
	if( nameChanged ) {
	    ShowAlgebra();
	}
    }


    public void OnSaveAs() {

	if( !m_editedAlgebra.renameAlgebra() ) return;
	ShowAlgebra();
    }

    public void OnClose() {

	if( !isEnabled() ) return;
	if( !m_editedAlgebra.closeAlgebra() ) return;
	m_UACalc.setEnabled(true);
	m_UACalc.requestFocus();
	dispose();
    }


//Operations menu callbacks

    public void OnEdit() {
	//Menu is diabled if m_NumOp == 0,
	//or if m_ThereIsACurrentAlgebra == false

	int n;
	
	OpChooser dialog = new OpChooser(
	    this, 
	    m_A,
	    "Select an operation to edit its values.",
	    0);
	n = dialog.getSelectedOp();
	if(n == -1) return;//User cancelled

	Operation f = m_A.getOp(n);
	valEdit = new ValueEditor(this, f);
    }

    public void OnAdd() {

	int n;

	if(m_NumOp > 0) {
	    ewkGetNumberDialog dialog = new ewkGetNumberDialog(
		this, 
		"Enter the number of the new operation.",
		m_NumOp, 0, m_NumOp);
	    n = dialog.getValidatedNumber();
	    if(n == -1) return;
	} else {
	    //It can only be f0.
	    n = 0;
	}
	Operation f = new Operation(m_A.getSize(), n);
	if( !f.Input(this) ) return;
	m_A.addOp(n, f);

	opList.write(m_A);
	m_NumOp += 1;
	setOpAble();
	if(m_NumOp < Configure.DEFAULT_NUMBER_OF_VISIBLE_OPERATIONS) {
	    pack();
	}
	setAlgebraModified(true);
    }

    public void OnRemove() {
	//Menu is diabled if m_NumOp = 0,
	//so we assume that m_NumOp > 0.

	int n;
	
	OpChooser dialog = new OpChooser(
	    this, 
	    m_A,
	    "Select an operation to remove.",
	    0);
	n = dialog.getSelectedOp();
	if(n == -1) return;//User cancelled

	int ans = JOptionPane.showConfirmDialog(
	    this, 
	    "Are you sure you want to remove operation "
	    + m_A.getOp(n).getNameString() + "? ",
	    "Confirm remove",
	    JOptionPane.YES_NO_OPTION,
	    JOptionPane.WARNING_MESSAGE);

	if(ans != JOptionPane.YES_OPTION) return;
	m_A.deleteOp(n);
	opList.write(m_A);
	m_NumOp -= 1;
	setOpAble();
	setAlgebraModified(true);
    }

    public void OnSwitch() {
	//Menu is diabled if m_NumOp <= 1,
	//so we assume that m_NumOp > 1.

	int n, m;
	
	if(m_NumOp == 2) {
	    n = 0;
	    m = 1;
	} else {
	    OpChooser dialog = new OpChooser(
		this, 
		m_A,
		"Select operation for switching.",
		0);
	    n = dialog.getSelectedOp();
	    if(n == -1) return;//User cancelled
	    
	    dialog = new OpChooser(
		this, 
		m_A,
		"Select operation to switch with " +
		m_A.getOp(n).getNameString() + ".",
		0);
	    m = dialog.getSelectedOp();
	    if(m == -1) return;//User cancelled
	    if( n == m ) {
		JOptionPane.showMessageDialog(
		    this,
		    "You have selected the same operation " +
		    m_A.getOp(n).getNameString() + " twice.\n",
		    "Nothing to be done",
		    JOptionPane.INFORMATION_MESSAGE);
		return;
	    }
	}
	int ans = JOptionPane.showConfirmDialog(
	    this, 
	    "Are you sure you want to switch operations "
	    + m_A.getOp(n).getNameString() + " and " +
	    m_A.getOp(m).getNameString() + "? ",
	    "Confirm switch",
	    JOptionPane.YES_NO_OPTION,
	    JOptionPane.WARNING_MESSAGE);

	if(ans != JOptionPane.YES_OPTION) return;
	m_A.switchOp(n, m);
	opList.write(m_A);
	setAlgebraModified(true);
    }

    public void OnChange() {
	//Menu is diabled if m_NumOp == 0,
	//so we assume that m_NumOp > 1.

	JOptionPane.showMessageDialog(
	    this,
	    "This feature is not implemented yet.\n" +
	    "Please use Remove/Add instead.\n" +
	    "Email to ewkiss@cs.elte.hu if you need it.\n",
	    "Not implemented",
	    JOptionPane.INFORMATION_MESSAGE);
    }

    public void setAlgebraModified(boolean flag) {
	m_editedAlgebra.setAlgebraModified(flag);
    }

//Helper functions

//Algebra I/O functions

    private void ShowAlgebra() {

	m_A = m_editedAlgebra.getAlgebra();
	algebraPane.setAlgebra(m_A);
	m_NumOp = m_A.getNumOp();

	opList.write(m_A);
	m_ThereIsACurrentAlgebra = true;
	setOpAble();

	SaveAsMenuItem.setEnabled(true);

	leftPane.setVisible(true);
	rightPane.setVisible(true);

        pack();
    }

    public void hideAlgebra() {

	Dimension d = getSize();
	m_ThereIsACurrentAlgebra = false;
	SaveMenuItem.setEnabled(false);
	SaveAsMenuItem.setEnabled(false);
	setOpAble();

	leftPane.setVisible(false);
	rightPane.setVisible(false);
	setSize(d);
    }

/////////////////////////

    private void setOpAble() {
	//Enables/Disables the Operations menus
	//and OpList according to the value of
	//m_NumOp and m_ThereIsACurrentAlgebra.

	if(!m_ThereIsACurrentAlgebra ) {
	    EditMenuItem.setEnabled(false);
	    AddMenuItem.setEnabled(false);
	    RemoveMenuItem.setEnabled(false);
	    SwitchMenuItem.setEnabled(false);
	    ChangeMenuItem.setEnabled(false);
	    opList.setVisible(false);
	    return;
	}
	algebraPane.setAlgebra(m_A);
	if( m_NumOp >= Algebra.OP_NUM_LIMIT-1) {
	    AddMenuItem.setEnabled(false);
	} else {
	    AddMenuItem.setEnabled(true);
	}
	if (m_NumOp == 0) {
	    EditMenuItem.setEnabled(false);
	    RemoveMenuItem.setEnabled(false);
	    SwitchMenuItem.setEnabled(false);
	    ChangeMenuItem.setEnabled(false);
	    EditMenuItem.setEnabled(false);
	    opList.setVisible(false);
	    return;
	}
	//Now m_NumOp > 0
	if (m_NumOp == 1) {
	    SwitchMenuItem.setEnabled(false);
	} else {
	    SwitchMenuItem.setEnabled(true);
	}
	EditMenuItem.setEnabled(true);
	RemoveMenuItem.setEnabled(true);
	ChangeMenuItem.setEnabled(true);
	opList.setVisible(true);
    }

//private variables

    private JFrame m_UACalc;
    private int m_selected;
    private int m_NumOp;
    private Algebra m_A;
    private EditedAlgebra m_editedAlgebra;
    private boolean m_ThereIsACurrentAlgebra;

//MenuItems
    private JMenuItem NewMenuItem;
    private JMenuItem OpenMenuItem;
    private JMenuItem SaveMenuItem;
    private JMenuItem SaveAsMenuItem;                                                                        
    private JMenuItem CloseMenuItem;

    private JMenuItem EditMenuItem;
    private JMenuItem AddMenuItem;
    private JMenuItem RemoveMenuItem;
    private JMenuItem SwitchMenuItem;
    private JMenuItem ChangeMenuItem;


//Components
    private Container pane;
    private JPanel leftPane;
    private JPanel rightPane;
    private AlgebraDisplayerPane algebraPane;
    private OpListTable opList;

//Children
    ValueEditor valEdit;

}

