package uacalc.alg.algedit;

import uacalc.alg.Algebra;
import uacalc.util.Project;
import uacalc.util.ewkFileChooser;
import uacalc.util.ewkGetNumberDialog;

import java.io.*;
import javax.swing.*;

public class EditedAlgebra {

    public EditedAlgebra(JFrame editor, JMenuItem SaveMenuItem) {
	m_name = null;
	m_editor = editor;
	m_SaveMenuItem = SaveMenuItem;
	m_A = null;
    } 

    public int createNewAlgebra() {
	//Returns
	//  0 if user cancelled, and algebra stays
	//  1 if user cancelled, but the algebra
	//                  may have become invalid.
	//  2 if new algebra was created.

	if( !PrepareDropAlgebra() ) return(0);

	String fn =
	ewkFileChooser.getFile(m_editor, 
			       Project.getAlgebraFilter(),
			       "Name of the new algebra",
			       ewkFileChooser.NEW);
	if(fn == null) return(0);

	Project m_project = new Project(fn);
	if( !m_project.create(m_editor, null) ) return(0);
	//Name may have changed:
	fn = m_project.getName(Project.AlgebraType);

	m_A = null;
	m_name = null;
	m_AlgCanBeSavedwoConfirm = true;

	ewkGetNumberDialog dialog = new ewkGetNumberDialog(
	    m_editor, 
	    "Enter the size of the new algebra.",
	    -1, 1, Algebra.ALG_SIZE_LIMIT-1);
	int n = dialog.getValidatedNumber();
	if(n == -1) {
	    //User cancelled
	    return(1);
	}
	m_A = new Algebra(fn);
	m_A.setSize(n);
	m_name = fn;
	//Number of operations is automatically 0.
	WriteAlgebra();
	return(2);
    }

    public boolean openAlgebra() {
	//Returns false on cancel.

	if( !PrepareDropAlgebra() ) return(false);

	//Get the NAME of the algebra to open.
	String fn = 
	    ewkFileChooser.getFile(m_editor, 
				   Project.getAlgebraFilter(),
				   "Algebra to open",
				   ewkFileChooser.OPEN);
	if(fn == null) return(false);

	//Check if this algebra is the one already open
	if( fn.equals(m_name) ) return(false);

	Algebra A = new Algebra(fn);
	if( !A.ReadAlgebra() ) {
	    //Unsuccesful read
	    return(false);
	}
	m_A = A;
	m_name = fn;
	setAlgebraModified(false);
	m_AlgCanBeSavedwoConfirm = false;
	return(true);
    }

    public boolean prepareWriteAlgebra() {
	//Returns true if algebra has been renamed.

	if( !m_AlgHasChanged ) return(false);
	if( m_AlgCanBeSavedwoConfirm ) return(false);
	//This happens, when the user opened an existing algebra,
	//modified it, and is about to save for the first time.
	Project m_project = new Project(m_name);
	if( !m_project.create(m_editor, null) ) return(false);
	//Name may have changed:
	m_name = m_project.getName(Project.AlgebraType);
	m_A.setName(m_name);
	return(true);
    }

    public boolean WriteAlgebra() {
	//Returns false if write error.

	try {
	    FileOutputStream fout =
		new FileOutputStream(m_name);
	    PrintStream fp =
		new PrintStream(fout);
	    m_A.Write(fp);
	    fp.close();
	    fout.close();
	}
	catch(Exception e) {
	    System.out.println("Save unsuccesful:\n" +
			       e.toString());
	    return(false);
	}
	setAlgebraModified(false);
	m_AlgCanBeSavedwoConfirm = true;
	return(true);
    }

    public boolean renameAlgebra() {
	//Returns false if nothing is to be done,
	//true if a new algebra has been cloned.

	//Get the new NAME of the algebra.
	String fn = 
	    ewkFileChooser.getFile(m_editor, 
				   Project.getAlgebraFilter(),
				   "New name of algebra",
				   ewkFileChooser.SAVE_AS);
	if(fn == null) return(false);

	if( fn.equals(m_name) ) {
	    //The new name is the same as the old one.
	    //No need to repaint, but we have to save it
	    //if it has been modified.
	    if( !m_AlgHasChanged ) return(false);
	    if( m_AlgCanBeSavedwoConfirm ) {
		WriteAlgebra();
		return(false);
	    } 
	    //If we are here, then the user opened an existing algebra,
	    //modified it, but has not saved it yet, and tried to
	    //saveAs it using its old name. This gets the same tratment
	    //as if a really new name had been typed.
	}

	Project m_project = new Project(fn);
	if( !m_project.create(
	    m_editor, 
	    "This algebra is cloned from " +
	    m_name + " .\n") ) return(false);
	//Name may have changed:
	fn = m_project.getName(Project.AlgebraType);

	m_A.setName(fn);
	m_name = fn;
	WriteAlgebra();
	return(true);
    }

    public boolean closeAlgebra() {
	if( !PrepareDropAlgebra() ) return(false);
	return(true);

    }

    private boolean PrepareDropAlgebra() {
	//Save the algebra if user wants to.
	//Algebra stays on the screen and is completely editable
	//after a call to this function.

	//Returns true if algebra can be dropped now,
	//false if algebra should not be dropped. That is:
	// false if user cancels the operation;
	// false if write error and user wants to keep the algebra;
	// true otherwise (algebra written succesfully,
	//   or the user does not want to save, or wants
	//   to drop the algebra despite an unsuccesful save attempt).


	if( m_A == null ) return(true);
	if( !m_AlgHasChanged ) return(true);

	int ans = JOptionPane.showConfirmDialog(
	    m_editor, 
	    "Algebra has been modified. Do you want to save it?",
	    "Save or abandon changes",
	    JOptionPane.YES_NO_CANCEL_OPTION,
	    JOptionPane.WARNING_MESSAGE);
	if(ans != JOptionPane.YES_OPTION &&
	   ans != JOptionPane.NO_OPTION) return(false);
	if(ans == JOptionPane.YES_OPTION) {
	    prepareWriteAlgebra();
	    //This may change the name.
	    if( !WriteAlgebra() ) { //Write error
		ans = JOptionPane.showConfirmDialog(
		    m_editor, 
		    "\nDrop algebra anyway?",
		    "Save error",
		    JOptionPane.YES_NO_OPTION,
		    JOptionPane.WARNING_MESSAGE);
		if(ans != JOptionPane.YES_OPTION) return(false);
	    }
	}
	return(true);
    }


    public Algebra getAlgebra() {
	return m_A;
    }

    public void setAlgebraModified(boolean flag) {
	m_AlgHasChanged = flag;
	m_SaveMenuItem.setEnabled(flag);
    }

    private String m_name;
    private JFrame m_editor;
    private JMenuItem m_SaveMenuItem;

    private Algebra m_A;
    private boolean m_AlgCanBeSavedwoConfirm;
    private boolean m_AlgHasChanged;
    
}
