package uacalc.alg.con;

import java.util.*;

public class MinimalSet {
    /**
     * minset is a SUBSET of the set on which
     * alpha and beta are NORMALIZED partitions.
     */
    //!!LATER merge the pretty printing routines with BasicPartition

    public MinimalSet(int[] minset, 
		      Partition alpha,
		      Partition beta) {
	m_minset = minset;
	Arrays.sort(m_minset);
	m_alpha = alpha;
	m_beta = beta;
    }

    /** Returns an ArrayList of int[]'s that are the.
     *  blocks of alpha restricted to set.
     *	It is assumed that part is normalized, and
     *  that set is sorted, and has no repetitions.
     *  Then the resulting blocks are also sorted.
     */
    private ArrayList makeBlocks(Partition part, int[] set) {
	
	ArrayList blocks[] = new ArrayList[part.getSize()];
	int i;
	for(i=0; i<set.length; i++) {
	    int r = part.representative(set[i]);
	    if (blocks[r] == null) blocks[r] = new ArrayList();
	    blocks[r].add(new Integer(set[i]));
	}

	ArrayList ans = new ArrayList();
	for(i=0; i<part.getSize(); i++) {
	    if( blocks[i] == null ) continue;
	    int[] next = new int[blocks[i].size()];
	    Iterator it = blocks[i].iterator();
	    int j=0;
	    while( it.hasNext() ) {
		next[j] = ((Integer)it.next()).intValue();
		j++;
	    }
	    ans.add(next);
	}
	return ans;
    }

    /**
     *  m_traces is an ArrayList of int[]'s.
     *  If [a,b,c,...] is an entry, then the corresponding
     *  trace consists of the alpha-blocks of a,b,c,....
     *  Also prepares a similar int[] m_tail, and
     *  m_alphaBlocks (see makeBlocks()).
     */
    private void makeTraces() {
	m_alphaBlocks = makeBlocks(m_alpha, m_minset);
	int[] factor = new int[m_alphaBlocks.size()];
	inv_factor = new int[m_alpha.getSize()];
	Iterator it = m_alphaBlocks.iterator();
	int i=0, j;
	while( it.hasNext() ) {
	    j = ((int[])it.next())[0];
	    factor[i] = j;
	    inv_factor[j] = i;
	    i++;
	}
	//factor is automatically sorted
	m_traces = makeBlocks(m_beta, factor);
	Iterator lit = m_traces.listIterator();
	ArrayList tail = new ArrayList();
	while( lit.hasNext() ) {
	    int [] next = (int[])lit.next();
	    if( next.length == 1 ) {
		tail.add(new Integer(next[0]));
		lit.remove();
	    }
	}
	m_tail = new int[tail.size()];
	it = tail.iterator();
	i = 0;
	while( it.hasNext() ) {
	    m_tail[i] = ((Integer)it.next()).intValue();
	    i++;
	}
    }

    private void classToString(int[] cls, StringBuffer sb) {
	String dash = "-";
	int i;
	for(i=0; i<cls.length; i++) {
	    if ( i != 0 ) {
		sb.append(dash);
	    }
	    sb.append(cls[i]);
	}
    }


    private void traceToString(int[] trace, StringBuffer sb) {
	String vert = "|";
	String openBracket = "[";
	String closeBracket = "]";
	sb.append(openBracket);
	int i;
	for(i=0; i<trace.length; i++) {
	    classToString( 
		(int[])m_alphaBlocks.get(
		    inv_factor[trace[i]]), sb);
	    if(i != trace.length-1) sb.append(vert);
	}
	sb.append(closeBracket);
    }


    public String toString() {
	makeTraces();
	String vert = "|";
	StringBuffer sb = new StringBuffer("");
	Iterator it = m_traces.iterator();
	while( it.hasNext() ) {
	    traceToString((int[])it.next(), sb);
	}
	int i;
	for(i=0; i<m_tail.length; i++) {
	    classToString( 
		(int[])m_alphaBlocks.get(
		    inv_factor[m_tail[i]]), sb);
	    sb.append(vert);
	}
	return(sb.toString());
    }


    private Partition m_alpha;
    private Partition m_beta;
    private ArrayList m_alphaBlocks;
    //inv_factor maps the representatives of the
    //m_alpha blocks to their index in ArrayList,
    //to speed up lookup.
    private int[] inv_factor;
    private int[] m_minset;
    private ArrayList m_traces = null;
    private int[] m_tail = null;

}
