package uacalc.alg.con;

/* Partition.java 2001/06/04 Ralph Freese */

import java.util.*;
import java.io.*;

/**
 * This interface specifies the basic operations for partitions on the set
 * {0, 1, ..., n-1}. This also specifies that the partition should have a
 * system of distinct representatives, an SDR.
 *
 * @author Ralph Freese
 * @version $Id:$
 */
public interface Partition {

  /**
   * Printing types: the internal representation.
   */
  public static final int INTERNAL = 0;

  /**
   * Printing types: the algebra program representtation: a comma separated
   * sequence of ints defining a map whose kernel is the partition.
   */
  public static final int EWK = 1;

  /**
   * Printing types: blocks, the useual way of writing a partition.
   */
  public static final int BLOCK = 2;

  /**
   * Printing types: blocks, plus number of blocks at the end.
   */
  public static final int HUMAN = 3;

  /**
   * This returns the array representation of the partition as described 
   * in Ralph Freese's notes on partitions, see 
   * {@link <a http://www.math.hawaii.edu/~ralph/Notes/}.
   */
  public int[] toArray();

  /** 
   * Note r and s must be roots and distinct.
   */
  public void joinBlocks(int r, int s);

  public Partition join(Partition part2);

  public Partition meet(Partition part2);

  public boolean leq(Partition part2);

  public void normalize();

  public int getSize();

  public int getNumberOfBlocks();

  public boolean isRelated(int i, int j);

  public String toString(int kind);

    /**
     * Returns a vector whose kernel is this partition,
     * and the values are block numbers ("Matt's format").
     * Needed, for example, to make a factor.
     */
  public int[] toMatt();
 
// rsf: new---added to Emil's stuff
 
//  public int root(int i);

//  public boolean isRoot(int i);
 
//  public int[] getRoots();
 
  public int representative(int i);

  public boolean isRepresentative(int i);
 
  public int[] getRepresentatives();
 
}

