package uacalc.alg.con.ui;

import uacalc.alg.Algebra;
import uacalc.alg.con.CongruenceLattice;
import uacalc.alg.con.Partition;
import uacalc.vectors.VectorList;
import uacalc.util.Project;
import uacalc.util.WholeNumberField;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;

/**
 * if isCancelled() returns false, then
 * getSelectedPartition() always works,
 * and getValidatedNumber works if congs.getSize() != -1
 */

public class CongruenceSelector extends JDialog {

    public CongruenceSelector(
	JFrame parent, String message,
	int value, 
  	  //value is the default value.
	  //set it to -1 to display the empty string initially.
	CongruenceLattice congs,
	boolean oneC,
	  //whether to display the OneCongruence button
	boolean zeroC,
	  //whether to display the ZeroCongruence button
	boolean generate,
	  //whether to display the Generate button. 
	boolean all
	  //whether to display the All button. 
	) {

        super(parent, true);
	//This doesn't work for some reason, the dialog is not
	//centered over this frame. A partial solution is the next command:
	setLocationRelativeTo(parent);

        setTitle("Select a congruence");
	m_parent = parent;
	m_value = value;
	m_A = congs.getAlgebra();
	m_congs = congs;
	m_zeroC = zeroC;
	m_oneC = oneC;
	m_generate = generate;

	m_size = congs.getSize();
	m_Partition = congs.getZeroCongruencePartition();
	isCancelled = false;
	zeroCongNum = -1;
	oneCongNum = -1;

	Object[] array;
	final WholeNumberField NumberField =
	    new WholeNumberField(m_value, 0);
	if ( m_size != -1) {
	    zeroCongNum = congs.getZeroCongruence();
	    oneCongNum = congs.getOneCongruence();
	    array = new Object[2];
	    array[0] = message;
	    array[1] = NumberField;
	} else {
	    array = new Object[1];
	    array[0] = message;
	}

        final String btnString1 = "Done";
        final String btnString2 = "Cancel";
        final String btnString3 = "Zero congruence";
        final String btnString4 = "One congruence";
        final String btnString5 = "Generate";
        final String btnString6 = "All";

	ArrayList buttons = new ArrayList();
	buttons.add(btnString1);
	buttons.add(btnString2);
	if( oneC ) buttons.add(btnString3);
	if( zeroC ) buttons.add(btnString4);
	if( generate ) buttons.add(btnString5);
	if( all ) buttons.add(btnString6);

        Object[] options = new Object[buttons.size()];
	int i;
	for( i=0; i<buttons.size(); i++ ) {
	    options[i]=buttons.get(i);
	}

        optionPane = new JOptionPane(array, 
				     JOptionPane.QUESTION_MESSAGE,
				     JOptionPane.OK_CANCEL_OPTION,
				     null,
				     options,
				     options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	optionPane.setValue(
	    JOptionPane.UNINITIALIZED_VALUE);

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                /*
                 * Instead of directly closing the window,
                 * we're going to change the JOptionPane's
                 * value property.
                 */
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
		}
		public void windowActivated(WindowEvent e) {
		    if( m_size != -1 ) {
			NumberField.requestFocus();
		    }
		}
        });

        NumberField.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    optionPane.setValue(btnString1);
		}
        });

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();

                if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object v = optionPane.getValue();

                    if (v == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }

                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (v.equals(btnString1)) {
			if( m_size == -1 ) {
			    isCancelled = true;
			    setVisible(false);
			} else {
			    m_value = NumberField.getValue();
			    if (m_value >= 0 && m_value < m_size) {
				// we're done; dismiss the dialog
				m_Partition = m_congs.getPartition(m_value);
				setVisible(false);
			    } else { 
				// number was invalid
				NumberField.selectAll();
				NumberField.requestFocus();
				JOptionPane.showMessageDialog(
				    CongruenceSelector.this,
				    "Please enter a number between " +
				    0 + " and " + (m_size-1) + ".",
				    "Number out of bounds",
				    JOptionPane.ERROR_MESSAGE);
			    }
			}
		    } else if (v.equals(btnString3)) {
			m_value = zeroCongNum;
			m_Partition = m_congs.getZeroCongruencePartition();
			setVisible(false);
		    } else if (v.equals(btnString4)) {
			m_value = oneCongNum;
			m_Partition = m_congs.getOneCongruencePartition();
			setVisible(false);
		    } else if (v.equals(btnString5)) {
			OnGenerate();
		    } else if (v.equals(btnString6)) {
			m_value = ALL;
			setVisible(false);
                    } else { 
                        // user closed dialog or clicked cancel
			isCancelled = true;
			setVisible(false);
                    }
                }
            }
        });
	pack();
	if( m_size != -1 )  NumberField.selectAll();
        setVisible(true);
    }

    public boolean isCancelled() {
	return isCancelled;
    }

    public int getValidatedNumber() {
        return m_value;
    }

    public Partition getSelectedPartition() {
        return m_Partition;
    }

    private void OnGenerate() {
	VectorList gen = new VectorList();
	int algSize = m_A.getSize();
	int[] limits = {algSize, algSize};
        gen.setLength(2);
        if( !gen.Input(m_parent, limits,
                        "Please enter generator") ) return;
	Project m_Project = new Project(m_A.getName());
	m_Project.appendToLog(
	    " [ Generating a congruence with the pairs\n"+
	    gen.toString());
	if( m_size == -1 ) {
	    m_Partition = m_congs.GeneratePartition(gen);
	    m_Project.appendToLog(
		" > The generated congruence is "+
		m_Partition.toString(Partition.HUMAN) + "\n");

	} else {
	    m_value = m_congs.Generate(gen);
	    m_Partition = m_congs.getPartition( m_value );
	    m_Project.appendToLog(
		" > The generated congruence is C"+ m_value + " = " +
		m_Partition.toString(Partition.HUMAN) + "\n");
	}
	setVisible(false);
    }

    private JOptionPane optionPane;
    private int m_value;
    private Partition m_Partition;

    private JFrame m_parent;
    private Algebra m_A;
    private int m_size;
    private CongruenceLattice m_congs;
    private boolean m_oneC;
    private boolean m_zeroC;
    private boolean m_generate;

    private int zeroCongNum;
    private int oneCongNum;
    private boolean isCancelled;
    public static final int ALL = -2;
}
