package uacalc.alg.opedit;

import uacalc.alg.Operation;
import uacalc.alg.OpArguments;

import uacalc.ex.TooManyArgumentsException;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;

public class OperationRendererList
    extends OperationRenderer {

    public OperationRendererList(Operation f,
				 ValueEditor editor,
				 Container pane) {
	super(f, editor, pane);

	m_ValueColumn = ValueEditorList.ValueColumn;
	m_NumValues = m_f.getNumValues();

	listModel = new OpValueListModel();
	OpValueList = 
	    new ValueEditorList(m_f, listModel);

	OpValueList.setTableCursorAt(0);

	m_pane.setLayout(new BoxLayout(m_pane, BoxLayout.X_AXIS));
	m_pane.add(OpValueList);

    }

    public void editValueAt(OpArguments args, int value) {
	OpValueList.editCellAt(
	    args.getNumValue(m_size), value);
    }

    public void editValueAt(OpArguments args) {
	OpValueList.editCellAt(
	    args.getNumValue(m_size), -2);
    }

    public void Update() {
	listModel.fireTableDataChanged();
	OpValueList.setFocusToEditedCell();
    }

    public boolean isEditorOpen() {
	return OpValueList.isEditorOpen();
    }

    private int m_NumValues;
    private int m_ValueColumn;
    private ValueEditorList OpValueList;
    private OpValueListModel listModel;

//inner classes

    class OpValueListModel extends AbstractTableModel {
	
	public int getColumnCount() {
	    return 2;
	}
    
	public int getRowCount() {
	    return m_NumValues;
	}
    
	public Object getValueAt(int row, int col) {
	    //This is obviously very inefficient.
	    //But the alternative is to store
	    //this possibly huge array of strings.
	
	    OpArguments args;
	    try {
		args = new OpArguments(m_arity);
	    } catch(TooManyArgumentsException e) {
		System.out.println(e.toString());
		return null;
	    }
	    args.setNumValue(row, m_size);
	    if(col == m_ValueColumn) {
		return new Integer(m_f.getValue(args));
	    }
	    String s = new String(" = " + args.toString(m_f));
	    //since m_ValueColumn = 0
	    return s;
	}
    
	public void setValueAt(Object value, int row, int col) {
	    //Cannot set String column at all
	    
	    OpArguments args;
	    try {
		args = new OpArguments(m_arity);
	    } catch(TooManyArgumentsException e) {
		System.out.println(e.toString());
		return;
	    }
	    args.setNumValue(row, m_size);
	    if(col == m_ValueColumn) {
		int intval = ((Integer)value).intValue();
		if( !m_f.tryValue(args, intval )) {
		    OnInvalidValue(args, intval);
		}
		fireTableCellUpdated(row, col);
		OperationChanged();
	    }
	}
	
	public Class getColumnClass(int col) {
	    if(col == m_ValueColumn ) return Integer.class;
	    return String.class;
	}
	
	public boolean isCellEditable(int row, int col) {
	    if(col == m_ValueColumn) return true;
	    return false;
	}
	
    }
    
}

