package uacalc.alg.opedit;

import uacalc.alg.Algebra;
import uacalc.alg.Operation;
import uacalc.alg.OpArguments;
import uacalc.alg.algedit.AlgebraEditor;
import uacalc.util.WholeNumberField;
import uacalc.config.Configure;

import uacalc.ex.TooManyArgumentsException;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.*;

public class ValueEditorList extends JScrollPane {

    public ValueEditorList(Operation f,
			   AbstractTableModel model) {

	m_f = f;
	m_model = model;
	m_size = f.getSize();
	m_arity = m_f.getArity();
	int i;

        table = new JTable(m_model);

//set up value fields
	NumberField =  
	    new WholeNumberField(0, Algebra.MAX_ALG_SIZE_DIGITS);
        DefaultCellEditor integerEditor =
            new DefaultCellEditor(NumberField) {
                //Override DefaultCellEditor's getCellEditorValue method
                //to return an Integer, not a String:
                public Object getCellEditorValue() {
                    return new Integer(NumberField.getValue());
                }
            };

        table.setDefaultEditor(Integer.class, integerEditor);

	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	table.setCellSelectionEnabled(true);
	table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	table.setTableHeader(null);
//	table.getTableHeader().setReorderingAllowed(false);
//	table.getTableHeader().setResizingAllowed(false);

//set up cell dimensions:
	int width = (int)NumberField.getPreferredSize().getWidth();
	int height = (int)NumberField.getPreferredSize().getHeight();
	Dimension cellDim = new Dimension(width, height);

//cell width
	TableColumn column = null;

	column = table.getColumnModel().getColumn(ValueColumn);
	column.setPreferredWidth(width);
	column.setMaxWidth(width);
	column.setMinWidth(width);

	int stringFieldWidth = getMaxArgStringWidth();
	column = table.getColumnModel().getColumn(StringColumn);
	column.setPreferredWidth(stringFieldWidth);
	column.setMaxWidth(stringFieldWidth);
	column.setMinWidth(stringFieldWidth);

//cell height
	table.setRowHeight(height);

//set renderer for String cells and column headers
	table.setDefaultRenderer(String.class,
				 new HeaderRenderer(cellDim));

//        for (i = 0; i < 2; i++) {
//            column = table.getColumnModel().getColumn(i);
//	    column.setHeaderRenderer(
//		new HeaderRenderer(cellDim));
//        }

//set up scrolling
	setViewportView(table);
	setMaxScrollSize();

//set preferred original size
	Dimension preferredTableSize = new Dimension(
	    (int)getMaximumSize().getWidth(),
	    Configure.DEFAULT_NUMBER_OF_VISIBLE_ELEMENTS*height);
	table.setPreferredScrollableViewportSize(preferredTableSize);

//put it to the upper left corner:
	setAlignmentY(Component.TOP_ALIGNMENT);
    }

    public void setTableCursorAt(int row) {

	table.requestFocus();
	table.setRowSelectionInterval(row, row);
	table.setColumnSelectionInterval(ValueColumn, ValueColumn);
    }

    public boolean isEditorOpen() {
	return table.isEditing();
    }

    public void setFocusToEditedCell() {

	WholeNumberField f = 
	    (WholeNumberField)(table.getEditorComponent());
	if ( f != null) f.requestFocus();
	else table.requestFocus();
    }


    public void editCellAt(int row, int value) {

	table.scrollRectToVisible(
	    table.getCellRect(row, ValueColumn, true));
	table.editCellAt(row, ValueColumn);
	WholeNumberField f = 
	    (WholeNumberField)(table.getEditorComponent());
	if(value != -2 ) f.setValue(value);
	f.requestFocus();
	f.selectAll();
	table.setRowSelectionInterval(row, row);
	table.setColumnSelectionInterval(ValueColumn, ValueColumn);
    }

    private int getMaxArgStringWidth() {
	//prepare a dummy label

	OpArguments args;
	try {
	    args = new OpArguments(m_arity);
	} catch(TooManyArgumentsException e) {
	    System.out.println(e.toString());
	    return -1;//impossible event anyway!!
	}
	int i;
	for(i=0; i<m_arity; i++) {
	    args.setValue(i, Algebra.ALG_SIZE_LIMIT-1);
	}
	String s = new String(" = " + args.toString(m_f));
	JLabel label = new JLabel(s);
	return (int)label.getPreferredSize().getWidth();
    }

    private void setMaxScrollSize() {
	/** Includes: column headers,
	 *  scrollbars, borders.
	 *  needed for the BoxLayout manager
	 */

	int w;
	int h;

	w = (int)table.getMaximumSize().getWidth();
	h = (int)table.getMaximumSize().getHeight();
//	h += (int)table.getTableHeader().getMaximumSize().getHeight();
        w += (int)getVerticalScrollBar().getMinimumSize().getWidth();
        h += (int)getHorizontalScrollBar().getMinimumSize().getHeight();
	Insets ins = getBorder().getBorderInsets(this);
	w += ins.left + ins.right;
	h += ins.top + ins.bottom;
        Dimension MaxScrollDim = new Dimension();
	MaxScrollDim.setSize(w, h);
	setMaximumSize(MaxScrollDim);
    }

    private JTable table;
    private AbstractTableModel m_model;
    private Operation m_f;
    private int m_size;
    private int m_arity;
    private WholeNumberField NumberField;
    public static final int ValueColumn = 0;
    public static final int StringColumn = 1;

//Inner classes

    class HeaderRenderer extends JLabel
	implements TableCellRenderer {
 
	HeaderRenderer(Dimension cellDim) {
	    JTable t = new JTable();
	    //Dummy table, to get default header attributes.

	    JTableHeader header = t.getTableHeader();
	    setOpaque(true);
	    setBorder(UIManager.getBorder("TableHeader.cellBorder"));
	    setHorizontalAlignment(LEFT);//since ValueColumn = 0
	    setForeground(header.getForeground());
	    setBackground(header.getBackground());
	    setFont(header.getFont());
	    setPreferredSize(cellDim);
	    setMaximumSize(cellDim);
	    setMinimumSize(cellDim);
	}
 
	public Component 
	getTableCellRendererComponent(JTable table, Object value, 
				      boolean isSelected, boolean hasFocus,
				      int row, int column) {
	    setText((value == null) ? "" : value.toString());
	    return this;
	}
    }

}

