package uacalc.vectors;

import uacalc.alg.Algebra;
import uacalc.config.Configure;
import uacalc.util.WholeNumberField;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;


public class VectorReader extends JDialog {

    public VectorReader(JFrame parent, int[] limits,
			VectorList list, String message ) {
	//It is assumed that limits contains integers.

        super(parent, true);
	//This doesn't work for some reason, the dialog is not
	//centered over this frame. A partial solution is the next command:
	setLocationRelativeTo(parent);
    
	m_limits = limits;
	m_length = limits.length;
	m_list = list;
	m_message = message;

	setTitle("Vector reader");
	m_index = 0;
	cancelled = false;
	isEmpty = true;

	prompt1 = new JLabel(m_message + " #" + m_index + ":");
	JLabel prompt2 = new JLabel("(use TAB to move between the fields)");
	JPanel pane = new JPanel();

	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));

	NumberField = new WholeNumberField[m_length];
	int i;
	JLabel label;
	for(i = 0; i< m_length; i++) {
	    label = new JLabel(i+ ":");
	    label.setAlignmentX(Component.LEFT_ALIGNMENT);
	    Configure.createBorder(label);
	    label.setMinimumSize(
		label.getPreferredSize());
	    label.setMaximumSize(
		label.getPreferredSize());
	    pane.add(label);
	    NumberField[i] = new WholeNumberField(
		-1,
		Algebra.MAX_ALG_SIZE_DIGITS);
	    NumberField[i].setAlignmentX(Component.LEFT_ALIGNMENT);
	    NumberField[i].setMinimumSize(
		NumberField[i].getPreferredSize());
	    NumberField[i].setMaximumSize(
		NumberField[i].getPreferredSize());
	    pane.add(NumberField[i]);
	}

	VectorBox = new JComboBox();
	VectorBox.addItem(emptyBoxString);
	VectorBox.setSelectedIndex(0);

        Object[] array = {prompt1,
			  prompt2,
			  pane,
			  "Vectors entered so far:",
			  VectorBox
	};

        final String btnString1 = "Next vector";
        final String btnString2 = "Finished";
        final String btnString3 = "Delete entry";
        final String btnString4 = "Cancel this";
        final String btnString5 = "Cancel all";
        Object[] options = {btnString1, btnString2, btnString3, 
			    btnString4, btnString5};

        optionPane = new JOptionPane(array, 
				     JOptionPane.QUESTION_MESSAGE,
				     JOptionPane.DEFAULT_OPTION,
				     null,
				     options,
				     options[0]);
        setContentPane(optionPane);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	optionPane.setValue(
	    JOptionPane.UNINITIALIZED_VALUE);

	focusWindow = 0;
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                /*
                 * Instead of directly closing the window,
                 * we're going to change the JOptionPane's
                 * value property.
                 */
                    optionPane.setValue(new Integer(
                                        JOptionPane.CLOSED_OPTION));
		}
		public void windowActivated(WindowEvent e) {
		    int j;
		    for(j=0; j< m_length; j++) {
			NumberField[j].selectAll();
		    }
		    NumberField[focusWindow].requestFocus();
		}
        });

	ActionListener ac = new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    optionPane.setValue(btnString1);
		}
	    };
	for(i=0; i< m_length; i++) {
	    NumberField[i].addActionListener(ac);
	}

        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();

                if (isVisible() 
                 && (e.getSource() == optionPane)
                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
                    Object v = optionPane.getValue();

                    if (v == JOptionPane.UNINITIALIZED_VALUE) {
                        //ignore reset
                        return;
                    }
                    // Reset the JOptionPane's value.
                    // If you don't do this, then if the user
                    // presses the same button next time, no
                    // property change event will be fired.
                    optionPane.setValue(
                            JOptionPane.UNINITIALIZED_VALUE);

                    if (v.equals(btnString1)) {
			OnNext();
                    } else if (v.equals(btnString2)) { 
			OnFinished();
                    } else if (v.equals(btnString3)) { 
			OnDelete();
                    } else if (v.equals(btnString4)) { 
			OnCancelLast();
		    } else {
			OnCancelAll();
                    }
                }
            }
        });
	pack();
        setVisible(true);
    }

    private boolean validateArgs() {
	int i;

	for(i=0; i<m_length; i++) {
	    int value = NumberField[i].getValue();
	    int lim = m_limits[i];
	    if (value < 0 || value >= lim) {
		// number was invalid
		JOptionPane.showMessageDialog(
		    VectorReader.this,
		    "Value " + i + " must be between " +
		    0 + " and " + (lim-1) + ".",
		    "Number out of bounds",
		    JOptionPane.ERROR_MESSAGE);
		focusWindow=i;
		return(false);
	    }
	}
//All values are valid
	int[] next = new int[m_length];
	for(i=0; i<m_length; i++) {
	    next[i] = NumberField[i].getValue();
	}
	addNewVector(next);
	return(true);
    }

    private void addNewVector(int[] next) {
	m_list.add(next);
	if( isEmpty ) {
	    VectorBox.removeItemAt(0);
	    isEmpty = false;
	}
	int last = m_list.size()-1;
	addItemToBox(last);
	VectorBox.setSelectedIndex(last);
    }

    private void addItemToBox(int index) {
	int[] next = (int[])m_list.get(index);
	VectorBox.addItem("#" + index + " = " +
			  IntArray.intArrayToString(next));
    }


    private void OnNext() {
	if ( !validateArgs() ) return;
	m_index++;
	prompt1.setText(m_message + " #" + m_index + ":");
	int i;
	for(i=0; i<m_length; i++) {
	    NumberField[i].setValue(-1);
	}
	focusWindow=0;
	NumberField[focusWindow].requestFocus();
    }

    private void OnFinished() {
	int i;

	boolean empty = true;
	for(i=0; i<m_length; i++) {
	    int value = NumberField[i].getValue();
	    if (value != -1 ) {
		empty = false;
		break;
	    }
	}
	if( !empty ) {
	    if ( !validateArgs() ) return;
	}
	//We are done
	setVisible(false);
    }

    private void OnDelete() {
	if( isEmpty ) return;
	int index = VectorBox.getSelectedIndex();
	if( index == -1 ) {
	    System.out.println("Internal error: invalid selection");
	    return;
	}
	String s = (String)VectorBox.getSelectedItem();
	int ans = JOptionPane.showConfirmDialog(
	    VectorReader.this, 
	    "Do you want to delete vector\n" + s + "?\n" +
	    "(If you want to delete another vector, then\n" +
	    "select it first and then press Delete again.)",
	    "Confirm",
	    JOptionPane.YES_NO_OPTION,
	    JOptionPane.QUESTION_MESSAGE);
	if(ans != JOptionPane.YES_OPTION) return;
	int last = m_list.size()-1;
	int i;
	for(i=index; i<= last; i++ ) {
	    VectorBox.removeItemAt(index);
	    //!Always remove index, because after every removal,
	    //the ComboBox has less and less entries!
	}
	m_list.remove(index);
	if( last == 0 ) {//There was only one item
	    VectorBox.addItem(emptyBoxString);
	    isEmpty = true;
	    return;
	}
	for(i=index; i<= last-1; i++ ) {
	    addItemToBox(i);
	}
	VectorBox.setSelectedIndex(last-1);//This is surely valid
    }

    private void OnCancelAll() {
	// user closed dialog or clicked cancelAll
	if( !isEmpty ) {
	    int ans = JOptionPane.showConfirmDialog(
		VectorReader.this, 
		"Are you sure you want to abandon all vectors entered so far?",
		"Confirm",
		JOptionPane.YES_NO_OPTION,
		JOptionPane.QUESTION_MESSAGE);
	    if(ans != JOptionPane.YES_OPTION) return;
	}
	m_list.clear();
	cancelled = true;
	setVisible(false);
    }

    private void OnCancelLast() {
	// user clicked cancelLast
	if( !isEmpty ) {
	    int ans = JOptionPane.showConfirmDialog(
		VectorReader.this, 
		"Do you want to stop entering this vector, and accept the rest?",
		"Confirm",
		JOptionPane.YES_NO_OPTION,
		JOptionPane.QUESTION_MESSAGE);
	    if(ans != JOptionPane.YES_OPTION) return;
	}
	setVisible(false);
    }

    public boolean isCancelled() {
	//Do not let the user enter an empty list??
	return cancelled || isEmpty;
    }

    WholeNumberField [] NumberField;
    private int m_length;
    private int[] m_limits;
    private VectorList m_list;
    private String m_message;
    private int m_index;
    private JLabel prompt1;
    private JOptionPane optionPane;
    private int focusWindow;
    private boolean cancelled;
    private JComboBox VectorBox;
    private boolean isEmpty;
    private String emptyBoxString = "None";
}
